#!/bin/bash
#
#   Copyright (C) 2010-2015 by Adaptive Computing Enterprises, Inc. All Rights Reserved.
#
#
#   Wraps the given command or a console application invocation into a valid Linux daemon.
#
#   Please note that the Insight application itself is a synchronous console application. 
#   When called from init.d script, the Insight must be invoked via this daemonizer script.

usagestring="Usage: $0 [--pidfile <path> ] [--workdir <path>] --user <username> -- <command> [<args> ...]"
pidfile=
workdir=
pid=

# The user to invoke the daemon from
user=

if [ -x "/sbin/runuser" ]; then
    SU="/sbin/runuser"
else
    SU="/bin/su"
fi

# redirect tty fds to /dev/null
redirectstd() {
    [[ -t 0 ]] && exec </dev/null
    [[ -t 1 ]] && exec >/dev/null
    [[ -t 2 ]] && exec 2>/dev/null
}
 
# close all non-std* fds
closefds() {
    eval exec {3..255}\>\&-
}
 
# daemonize
daemonize() {
    # fork
    (
        redirectstd
        # Do not react on HUP and INT signals
        trap '' 1 2
        # CWD should not be a mounted fs.
        if [ -n "$workdir" ]
        then
            cd "$workdir"
        else
            cd /
        fi
        # close unneeded file descriptors
        closefds
        if [[ $(type -t "$1") != file ]]
        then
            $SU - $user -c "$@"
        else
            exec $SU - $user -c "$@"
        fi
    ) &
    pid=$!
    # guard against HUP in parent
    disown -h $!
}

isalive() {
    [ -n "$pid" -a -e "/proc/$pid" ]
    return $?
}

checkpid() {
    # on slow systems this can cause false-positive result,
    # i.e. process can fail after this check, but we handle it as running.
    sleep 8
    isalive || return 1
    [ -n "$pidfile" ] && echo $pid > $pidfile
    return 0
}

# read program arguments until '--'
while [ "$1" != "--" -a "$#" -ne "0" ]
do
    case $1 in
        --pidfile)
            pidfile=$2
            shift 2
            rm -f $pidfile && touch $pidfile || {
                echo "Can't access $pidfile" >&2
                exit 1
            }
            ;;
        --user)
            user=$2
            shift 2
            ;;
        --workdir)
            workdir=$2
            shift 2
            cd $workdir || {
                echo "Can't change dir to $workdir" >&2
                exit 1
            }
            ;;
        *)
            echo "$usagestring" >&2
            exit 1
            ;;
    esac
done
# Skip '--'
if [ "$1" != "--" ]
then
    echo "$usagestring" >&2
    exit 1
fi
shift

. /opt/insight/etc/insight.conf
daemonize "JAVA_HOME=$JAVA_HOME $@" && checkpid && exit 0 || exit 1
