import ch.qos.logback.classic.encoder.PatternLayoutEncoder
import ch.qos.logback.core.rolling.FixedWindowRollingPolicy
import ch.qos.logback.core.rolling.RollingFileAppender
import ch.qos.logback.core.rolling.SizeBasedTriggeringPolicy

import static ch.qos.logback.classic.Level.*

// Scan the configuration file every 30 seconds and reload if changed
scan("30 seconds")

// Create appenders for multiple log files (used below)
['insight', 'messages', 'reportingFramework'].each { String name ->
	appender(name, RollingFileAppender) {
		file = "/opt/insight/log/${name}.log"
		append = true
		rollingPolicy(FixedWindowRollingPolicy) {
			fileNamePattern = "/opt/insight/log/${name}.log.%i"
			minIndex = 1
			maxIndex = 10
		}
		triggeringPolicy(SizeBasedTriggeringPolicy) {
			maxFileSize = "100MB"
		}
		encoder(PatternLayoutEncoder) {
			pattern = "%d{yyyy-MM-dd'T'HH:mm:ss.SSSZ}\t%t\t%p\t%c\t0\t\t%m%n"
		}
	}
}

// Log messages received by Insight over ZMQ to messages.log
logger("com.ace.insight.zmq.listener.MoabDataRoutingListener", INFO, ["messages"], false)
logger("com.ace.insight.zmq.listener.ControlProcessingListener", INFO, ["messages"], false)

// Log messages Insight sends to reporting framework over Kafka to reportingFramework.log
logger("com.ace.insight.logging.ReportingFrameworkLogger", INFO, ["reportingFramework"], false)

// All messages warning level and above (ERROR, etc) are logged
root(WARN, ["insight"])

// Specific loggers for more detailed messages
logger("com.ace.insight.app.Application", DEBUG, ["insight"], false)
logger("com.ace", INFO, ["insight"], false)
