#!/opt/insight/python/bin/python

# create-time-report.py
# Generates a CSV formatted file that contains logged Moab and Insight processing times
# for each iteration of Moab scheduling from Insight log files.
#
# This is for use in debugging what RMPOLLINTERVAL should be configured in Moab to let
# Insight finish processing each iteration without getting behind in processing.
#
# Usage: ./create-time-report.py <output-file>
#
# NOTE: This is provided without warranty and is used at your own risk.

import os
import re
import sys
from datetime import datetime

division_factor = 1000  # milliseconds -> seconds

if len(sys.argv) < 2:
    print 'Usage: ./create-time-report.py <output-file>'
    sys.exit(1)

path = os.path.dirname(os.path.realpath(__file__))
out_file_name = sys.argv[1]

date_format = '%Y-%m-%dT%H:%M:%S.%f'
starting_pattern = re.compile(
    '^([0-9\-:T.]+)\s+.*Moab iteration (\d+) which was started .* \((\d+) milliseconds ago\).*')
finished_pattern = re.compile('.*Moab iteration (\d+) which took (\d+) milliseconds .* in (\d+) milliseconds.*')
out_file = open(out_file_name, 'w')
out_file.write('Iteration,Elapsed Time,Moab Time,Insight Time,Start Delay\n')

iteration_delays = {}
iteration_times = {}
last_iteration = None
iteration_offset = 0
first_time = None
file_read = False
for i in range(9, -1, -1):
    file = '%s/../log/insight.log.%s' % (path, i)
    if file.endswith('.0'):
        file = file[0:-2]
    if not os.path.exists(file):
        continue
    file_read = True
    print 'Reading file %s' % os.path.basename(file)
    in_file = open(file)
    for line in in_file.readlines():
        if 'Started processing Moab iteration' in line:
            m = starting_pattern.match(line)
            if not m:
                continue
            iteration = m.group(2)
            iteration_delays[iteration] = m.group(3)
            # Set start time and/or find elapsed time
            # Strip off timezone since it is not supported on all platforms
            date = datetime.strptime(m.group(1)[:-5], date_format)
            if not first_time:
                first_time = date
            td = (date - first_time)
            iteration_times[iteration] = (td.microseconds + (
                td.seconds + td.days * 24.0 * 3600.0) * 10.0 ** 6.0) / 10.0 ** 6.0
        elif 'Finished processing Moab iteration' in line:
            m = finished_pattern.match(line)
            if not m:
                continue
            iteration = m.group(1)

            # If moab is reset, the iteration gets set back to 0
            #   this makes graphing hard, so just keep counting
            #   with the last iteration added to this one if that happens
            iteration_num = int(iteration)
            if iteration_num == 0 and last_iteration is not None:
                iteration_offset += last_iteration + 1

            # Get info from start iteration message
            if iteration in iteration_delays.keys():
                iteration_delay = iteration_delays.pop(iteration)
            else:
                iteration_delay = 0
            if iteration in iteration_times.keys():
                iteration_time = iteration_times.pop(iteration)
            else:
                iteration_time = ''

            # Write out line
            out_file.write('%s,%s,%f,%f,%f\n' % (
                iteration_num + iteration_offset, iteration_time, float(m.group(2)) / division_factor,
                float(m.group(3)) / division_factor, float(iteration_delay) / division_factor))

            # Set last iteration
            last_iteration = iteration_num
    in_file.close()

if not file_read:
    print 'Could not read from any Insight log files. Does the script exist in the utils subdirectory?'
out_file.close()
