import datetime
import json
import uuid

import zmq

from lib.aes import encrypt_message, decrypt_message

_context = None


def get_context():
    global _context
    if _context == None:
        raise Exception('The ZMQ context has not been initialized; please initialize the context first')
    return _context


def setup_context():
    global _context
    _context = zmq.Context.instance()


def teardown_context():
    global _context
    _context.destroy(linger=0)


def get_socket(socket_type):
    socket = get_context().socket(socket_type)
    socket.RCVTIMEO = 2000
    socket.setsockopt(zmq.LINGER, 0)
    return socket


def create_message(message_type, message_body, encrypt=True):
    message = json.dumps({
        'messageId': str(uuid.uuid1()),
        'senderId': "system-tests@localhost",
        'sentDate': datetime.datetime.now().isoformat(),
        'ttl': 10,
        'messageType': message_type,
        'body': message_body
    }, separators=(',', ':'))
    if not encrypt:
        return message
    return encrypt_message(message)


def get_response(socket, decrypt=True):
    try:
        response_string = socket.recv()
        if not decrypt:
            return json.loads(response_string)
        return json.loads(decrypt_message(response_string))
    except zmq.ZMQError:
        raise Exception(
            'Did not receive response from socket within the timeout, the other side of the socket is probably not running')
