__author__ = 'jpratt'
import re
import sys

import pymongo

from lib.util import get_regex_value


def add_mongo_parser_arguments(config, parser):
    mongo_username = get_regex_value(re.search('mongo.username(.*?)=(.*?)"(.*?)"', config), 3, 'insight_user')
    mongo_password = get_regex_value(re.search('mongo.password(.*?)=(.*?)"(.*?)"', config), 3, 'secret4')
    mongo_host = get_regex_value(re.search('mongo.host(.*?)=(.*?)"(.*?)"', config), 3, 'localhost')
    mongo_port = int(get_regex_value(re.search('mongo.port(.*?)=\s*(\d*) ', config), 2, '27017'))
    mongo_database_name = get_regex_value(re.search('mongo.databaseName(.*?)=(.*?)"(.*?)"', config), 3, 'insight')

    parser.add_argument('--mongo-database-name',
                        default=mongo_database_name,
                        help="The MongoDB database name to use"
                        )
    parser.add_argument('--mongo-username',
                        default=mongo_username,
                        help="The username to use when connecting to MongoDB"
                        )
    parser.add_argument('--mongo-password',
                        default=mongo_password,
                        help="The password to use when connecting to MongoDB"
                        )
    parser.add_argument('--mongo-host',
                        default=mongo_host,
                        help="The MongoDB host to use"
                        )
    parser.add_argument('--mongo-port',
                        default=mongo_port,
                        help="The MongoDB port to use"
                        )
    return parser


def delete_nodes_from_mongo(insight, args):
    insight.node_samples. delete_many({'nodeName': {'$in': args['object_names']}})
    insight.nodes_auxiliary.delete_many({'name': {'$in': args['object_names']}})
    insight.node_history.delete_many({'name': {'$in': args['object_names']}})
    insight.node_life_cycle_journal.delete_many({'nodeName': {'$in': args['object_names']}})
    insight.resource_job_timeline_jobs_per_node.delete_many({'node_id': {'$in': args['object_names']}})
    insight.resource_job_timeline_node_list_with_job_count.delete_many({'node_id': {'$in': args['object_names']}})
    insight.resource_job_timeline_node_state_transitions.delete_many({'node_id': {'$in': args['object_names']}})
    insight.resource_job_timeline_reservations_per_node.delete_many({'node_id': {'$in': args['object_names']}})
    insight.nodes.delete_many({'name': {'$in': args['object_names']}})


def delete_jobs_from_mongo(insight, args):
    insight.job_samples.delete_many({'jobName': {'$in': args['object_names']}})
    insight.job_state_journal.delete_many({'jobName': {'$in': args['object_names']}})
    insight.jobs.delete_many({'name': {'$in': args['object_names']}})
    insight.resource_job_timeline_jobs_per_node.delete_many({'job_id': {'$in': args['object_names']}})
    insight.resource_job_timeline_node_list_with_job_count.delete_many({'job_id': {'$in': args['object_names']}})
    insight.resource_job_timeline_procs_per_job.delete_many({'job_id': {'$in': args['object_names']}})
    insight.workload_view.delete_many({'job_id': {'$in': args['object_names']}})


class MongoConnection():
    def __init__(self, args):
        self.client = pymongo.MongoClient(args['mongo_host'], args['mongo_port'])
        self.insight = self.client[args['mongo_database_name']]
        if args['mongo_username']:
            try:
                self.insight.authenticate(args['mongo_username'], args['mongo_password'])
            except:
                sys.exit("Mongo authentication failed for user '%s' to database '%s' on host '%s:%s'" % (
                    args['mongo_username'],
                    args['mongo_database_name'],
                    args['mongo_host'],
                    args['mongo_port']))

    def withConnection(self, action, args):
        try:
            action(self.insight, args)
        finally:
            self.client.close()
