import signal
import time


def handle_timeout_wait_for(signum, frame):
    raise Exception('Timed out while waiting for function to complete successfully')


def wait_for(function, *args, **kwargs):
    '''Waits for the passed in function to pass by returning a value that evaluates to 'true'.
       Handles exceptions correctly as well.
       This function does not raise exceptions (unless input parameters are wrong)
       but instead returns the return value from the function or False depending on if the condition succeeded or not.
       Pass 'wait_timeout' or 'wait_sleep' to change the default
       time parameters used.
       'wait_reverse' may be used to return successfully when an error condition is encountered or the function returns unsuccessfully.
       All other args are passed to the function.'''
    timeout = 30
    sleep = 1
    reverse = False
    if 'wait_timeout' in kwargs.keys():
        timeout = kwargs.pop('wait_timeout')
    if 'wait_sleep' in kwargs.keys():
        sleep = kwargs.pop('wait_sleep')
    if 'wait_reverse' in kwargs.keys():
        sleep = kwargs.pop('wait_reverse')
    if not timeout or timeout <= 0:
        raise Exception('"The timeout for wait_for must be greater than 0')
    if not sleep or sleep < 0:
        raise Exception('The sleep for wait_for must be greater or equal to 0')
    print 'Using timeout of %s seconds' % timeout
    try:
        signal.signal(signal.SIGALRM, handle_timeout_wait_for)
        signal.alarm(timeout)
        ret = None
        while (reverse and ret) or not ret:
            try:
                ret = function(*args, **kwargs)
            except Exception as e:
                if 'Timed out' in str(e):
                    signal.alarm(0)
                    raise e
                if reverse:
                    break
            if (reverse and ret) or not ret:
                print 'Waiting for %s to return successfully...' % function.__name__
                time.sleep(sleep)
        signal.alarm(0)
        print 'Finished waiting for %s successfully' % function.__name__
        return ret
    except Exception as e:
        print 'Timed out while waiting for function %s after %d seconds' % (function.__name__, timeout)
        return False


import sys


def query_yes_no(question, default="yes"):
    """Ask a yes/no question via raw_input() and return their answer.

    "question" is a string that is presented to the user.
    "default" is the presumed answer if the user just hits <Enter>.
        It must be "yes" (the default), "no" or None (meaning
        an answer is required of the user).

    The "answer" return value is True for "yes" or False for "no".
    """
    valid = {"yes": True, "y": True, "ye": True,
             "no": False, "n": False}
    if default is None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("invalid default answer: '%s'" % default)

    while True:
        sys.stdout.write(question + prompt)
        choice = raw_input().lower()
        if default is not None and choice == '':
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            sys.stdout.write("Please respond with 'yes' or 'no' "
                             "(or 'y' or 'n').\n")


def get_regex_value(match, index, default_value=None):
    try:
        return match.group(index) or default_value
    except:
        if default_value:
            return default_value
        else:
            return "not_found_in_config"
