#!/opt/insight/python/bin/python

# ping.py
# Pings the Insight daemon and returns whether it is responding to messages or not.    This is helpful
# in determining if Insight is fully running or is still initializing.
#
# Insight defaults are used in this script
#
# Usage: ./ping.py [<retry_count>]
# If no retry_count is provided, the call will succeed or fail based on a single query.
#
# NOTE: This is provided without warranty and is used at your own risk.
import sys

from lib import ace_zmq
from lib.control import send_control_message, command_ping
from lib.util import wait_for


def wait_for_ping(retry_count):
    ace_zmq.setup_context()
    try:
        # No waiting if retry_count is 0
        if retry_count == 0:
            send_control_message(command_ping)
        else:
            wait_for(send_control_message, command_ping, retry_count=retry_count, delay=1)
        print 'Successfully received ping response from Insight'
        return 0
    except Exception as e:
        print str(e)
        return 1
    finally:
        ace_zmq.teardown_context()


def main():
    retry_count = 0
    if len(sys.argv) > 1:
        retry_count = int(sys.argv[1])
    sys.exit(wait_for_ping(retry_count))


if __name__ == '__main__':
    main()
