#!/opt/insight/python/bin/python

# sync.py
# This script is used when insight's job database is out of sync with Moab's job database.
# Insight will update any job that is no longer in Moab with a 'Completed' job state.
# It also looks for job state changes if the job exists in both databases and will update the Insight database if different.
# All jobs in Moab that are not yet in Insight will be updated as well.
#
# Insight defaults are used in this script
#
# Usage: ./sync.py [<retry_count>]
# If no retry_count is provided, the call will succeed or fail based on a single query.
#
# NOTE: This is provided without warranty and is used at your own risk.
import sys

from lib import ace_zmq
from lib.control import send_control_message, command_sync
from lib.util import wait_for


def wait_for_sync(retry_count):
    ace_zmq.setup_context()
    try:
        # No waiting if retry_count is 0
        if retry_count == 0:
            send_control_message(command_sync)
        else:
            wait_for(send_control_message, command_sync, retry_count=retry_count, delay=1)
        print 'Successfully received sync response from Insight'
        return 0
    except Exception as e:
        print str(e)
        return 1
    finally:
        ace_zmq.teardown_context()


def main():
    retry_count = 0
    if len(sys.argv) > 1:
        retry_count = int(sys.argv[1])
    sys.exit(wait_for_sync(retry_count))


if __name__ == '__main__':
    main()
